@@ -25,7 +25,7 @@
 import os
 from jupyterhub.handlers import BaseHandler
 from jupyterhub.auth import Authenticator
-from tornado import web
+from tornado import web, gen
 from traitlets import Unicode
 from ldap3 import Connection, Server, ALL
 import jwt
@@ -35,6 +35,7 @@ import requests
 
 JUPYTERHUB_COU = os.getenv('FABRIC_COU_JUPYTERHUB', 'CO:COU:Jupyterhub:members:active')
 CILOGON_JWKS_URL = os.getenv('CILOGON_JWKS_URL')
+VOUCH_COOKIE_NAME = os.getenv('VOUCH_COOKIE_NAME')
 
 
 class VouchProxyLoginHandler(BaseHandler):
@@ -68,6 +69,10 @@ class VouchProxyLoginHandler(BaseHandler):
             self.log.warn("FABRIC user {} is not in {}".format(user_name, JUPYTERHUB_COU))
             raise web.HTTPError(403, "Access not allowed")
 
+        if self.get_cookie(VOUCH_COOKIE_NAME):
+            self.log.warn("Vouch Cookie is expired")
+            raise web.HTTPError(403, "Access not allowed")
+
         userdict = {"name": user_name}
         userdict["auth_state"] = auth_state = {}
         auth_state['id_token'] = id_token
@@ -126,8 +131,7 @@ class VouchProxyLoginHandler(BaseHandler):
         conn.unbind()
         return attributes
 
-    @staticmethod
-    def verify_id_token(id_token):
+    def verify_id_token(self, id_token):
         """
         verify signature and expiration date
 
@@ -148,10 +152,10 @@ class VouchProxyLoginHandler(BaseHandler):
 
             options = {'verify_aud': False}
             claims = jwt.decode(id_token, key=key, verify=True, algorithms=['RS256'], options=options)
-            print("Decoded Identity Token %s", json.dumps(claims))
+            self.log.debug("Decoded Identity Token %s", json.dumps(claims))
             return True
         except Exception as ex:
-            print(ex)
+            self.log.error(ex)
 
         return False
 
@@ -160,8 +164,6 @@ class VouchProxyAuthenticator(Authenticator):
     """
     Accept the authenticated user name from the X-Vouch-User HTTP header.
     """
-    isMemberOf = 'isMemberOf'
-
     user_header_name = Unicode(
         default_value='X-Vouch-User',
         config=True,
@@ -182,10 +184,14 @@ class VouchProxyAuthenticator(Authenticator):
         config=True,
         help="""HTTP header to inspect for the authenticated user's Refresh Token.""")
 
-    #def get_handlers(self, app):
-    #    return [
-    #        (r'/login', VouchProxyLoginHandler),
-    #    ]
+    def get_handlers(self, app):
+        return [
+            (r'/login', VouchProxyLoginHandler),
+        ]
+
+    @gen.coroutine
+    def authenticate(self, *args):
+        raise NotImplementedError()
 
     async def pre_spawn_start(self, user, spawner):
         """ Populate credentials to spawned notebook environment
@@ -210,67 +216,8 @@ class VouchProxyAuthenticator(Authenticator):
         """
         self.log.debug("[refresh_user] always trigger refresh authentication")
         await handler.stop_single_user(user, user.spawner.name)
+        handler.clear_cookie("fabric-service")
         handler.clear_cookie("jupyterhub-hub-login")
         handler.clear_cookie("jupyterhub-session-id")
         handler.redirect('/hub/logout')
-        return True
-
-    async def authenticate(self, handler, data=None):
-        self.log.debug("I am in authenticate function")
-        user_name = handler.request.headers.get(self.user_header_name, None)
-        self.log.debug("I am in authenticate function: {}".format(user_name))
-        if user_name is None:
-            self.log.error("Missing User Name")
-            raise web.HTTPError(401)
-
-        cookie = handler.request.headers.get(self.cookie_header_name, None)
-        self.log.debug("I am in authenticate function: {}".format(cookie))
-        if cookie is None:
-            self.log.error("Missing Vouch Cookie")
-            raise web.HTTPError(401)
-
-        refresh_token = handler.request.headers.get(self.refresh_token_header_name, None)
-        self.log.debug("I am in authenticate function: {}".format(refresh_token))
-        if refresh_token is None:
-            self.log.error("Missing Refresh Token")
-            raise web.HTTPError(401)
-
-        id_token = handler.request.headers.get(self.id_token_header_name, None)
-        self.log.debug("I am in authenticate function: {}".format(id_token))
-        if id_token is None:
-            self.log.error("Missing Identity Token")
-            raise web.HTTPError(401)
-
-        if not VouchProxyLoginHandler.verify_id_token(id_token=id_token):
-            self.log.error("Signature has expired")
-            raise web.HTTPError(403, "Signature has expired")
-
-        if not self.is_in_allowed_cou(username=user_name):
-            self.log.warn("FABRIC user {} is not in {}".format(user_name, JUPYTERHUB_COU))
-            raise web.HTTPError(403, "Access not allowed")
-
-        userdict = {"name": user_name}
-        userdict["auth_state"] = auth_state = {}
-        auth_state['id_token'] = id_token
-        auth_state['refresh_token'] = refresh_token
-        auth_state['cookie'] = cookie
-
-        return userdict
-
-    def is_in_allowed_cou(self, username):
-        """ Checks if user is in Comanage JUPYTERHUB COU.
-
-            Args:
-                username: i.e. ePPN
-
-            Returns:
-                Boolean value: True if username has attribute of JUPYTERHUB_COU, False otherwise
-        """
-        attributelist = VouchProxyLoginHandler.get_ldap_attributes(username)
-        if attributelist:
-            self.log.debug("attributelist acquired.")
-            if attributelist[self.isMemberOf]:
-                for attribute in attributelist[self.isMemberOf]:
-                    if attribute == JUPYTERHUB_COU:
-                        return True
-        return False
\ No newline at end of file
+        return True
\ No newline at end of file
